#include "studentinfo.h"
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qxmlstream.h>
#include <qmessagebox.h>
#include <qinputdialog.h>


studentinfo::studentinfo(QWidget *parent)
    : QWidget(parent)
{
    ui.setupUi(this);  // dodaju se elementi

    //pravi se regularni izraz kojim se proverava da li je za JMBG uneto 13 cifara
    QRegExp rx("[0-9]{13}");
    ui.jmbgEdit->setValidator(new QRegExpValidator(rx, this));

}

studentinfo::~studentinfo()
{}

// metod (slot) koja se izvrsava kada se klikne (emituje signal) na dugme saveButton
void studentinfo::saveButtonClicked()
{
    // proverava se da li JMBG ima 13 cifara, ako nema izdaje se poruka
    if(!ui.jmbgEdit->hasAcceptableInput())
    {
        QMessageBox::warning(this, "Neispravan JMBG", "JMBG mora da sadrzi 13 cifara", QMessageBox::Ok);
    }
    else
    {
        //zadaje se datoteka u koju ce se sacuvati podaci
        QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"),"", tr("Text files (*.txt);; XML files(*.xml)"));

        //zavisno od ekstenzije zadate datoteke poziva se odgovarajuci metod
        if (fileName.endsWith("txt"))
            writeTxtFile(fileName);
        else
            writeXMLFile(fileName);
    }
}

// metod (slot) koji se izvrsava kada se klikne (emituje signal) na dugme dodajButton
void studentinfo::on_addButton_clicked()
{
    //prikazuje se dijalog za unos naziva predmeta
    QString course=QInputDialog::getText(this, "Prijava ispita", "Naziv predmeta");

    // ako je unet naziv predmeta, dodaje se u listu prijavljenih predmeta
    if(!course.isEmpty())
    {
        QListWidgetItem *item=new QListWidgetItem(course, ui.coursesList);
        ui.coursesList->setCurrentItem(item);
    }
}

//metod (slot) koji se izvrsava kada se klikne na dugme obrisiButton -
//brise se izabrani predmet iz liste
void studentinfo::on_deleteButton_clicked()
{
    QListWidgetItem *curItem = ui.coursesList->currentItem();

    if (curItem)  // ako je izabran neki predmet
    {
        ui.coursesList->takeItem( ui.coursesList->currentRow());
        delete curItem;

        // ako postoji jos predmeta u listi, prvi postaje oznacen
        if (ui.coursesList->count() > 0)
            ui.coursesList->setCurrentRow(0);
    }
}

//metod (slot) koji se izvrsava kada se klikne na dugme obrisiSveButton -
//brisu se svi predmeti iz liste prijavljenih predmeta
void studentinfo::on_deleteAllButton_clicked()
{
    ui.coursesList->clear();
}

//metod (slot) koji se izvrsava kada se klikne na dugme openButton -
//unose se podaci o studentu iz datoteke
void studentinfo::openButtonclicked()
{
    //odredjuje se datoteka preko QFileDialoga
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),"", tr("Text files (*.txt);; XML files(*.xml)"));

    //zavisno od ekstenzije izabrane datoteke poziva se odgovarajuci metod
    //za citanje podataka o studentu
    if (fileName.endsWith("txt"))
        readTxtFile(fileName);
    else
        readXMLFile(fileName);
}

//metod koji cita podatke o studentu iz tekstualne datoteke
void studentinfo::readTxtFile(QString fileName)
{
    QFile file(fileName);

    if(!file.open(QIODevice::ReadOnly))
    {
        QMessageBox::warning(this, "Ucitavanje podataka o studentu", "Ne moze da se otvori datoteka", QMessageBox::Ok);
    }
    else
    {
        QString line;
        ui.coursesList->clear();

        QTextStream t(&file);
        line=t.readLine();
        ui.nameEdit->setText(line);

        line=t.readLine();
        ui.surnameEdit->setText(line);

        line=t.readLine();
        ui.indexEdit->setText(line);

        line=t.readLine();
        ui.jmbgEdit->setText(line);

        line=t.readLine();

        // ako je student upisan, citaju se predmeti i unose u listu prijavljenih ispita
        if(line.compare("upisan")==0)
        {
            ui.enrolledButton->setChecked(true);
            while ( !t.atEnd() )
            {
                line = t.readLine();
                QListWidgetItem *item=new QListWidgetItem(line, ui.coursesList);
            }
            ui.coursesList->setCurrentRow(ui.coursesList->count()-1);

        }
        else
            ui.notEnrolledButton->setChecked(true);

        file.close();
    }
}

//metod koji cita podatke o studentu iz XML datoteke
void studentinfo::readXMLFile(QString fileName)
{
    QFile file(fileName);

    if(!file.open(QIODevice::ReadOnly))
    {
        QMessageBox::warning(this, "Ucitavanje podataka o studentu", "Ne moze da se otvori datoteka", QMessageBox::Ok);
    }
    else
    {
        ui.coursesList->clear();
        QXmlStreamReader reader(&file);

        QString tekst;
        while (!reader.atEnd())
        {
            reader.readNext();

            if (reader.isStartElement())
            {
                if (reader.name() == "ime")
                {
                    reader.readNext();

                    if(reader.isCharacters())
                        ui.nameEdit->setText(reader.text().toString());
                }

                if (reader.name() == "prezime")
                {
                        reader.readNext();

                        if(reader.isCharacters())
                        {
                            ui.surnameEdit->setText(reader.text().toString());
                        }
                }


                if (reader.name() == "brojIndeksa")
                {
                        reader.readNext();

                        if(reader.isCharacters())
                        {
                            ui.indexEdit->setText(reader.text().toString());
                        }
                }


                if (reader.name() == "JMBG")
                {
                        reader.readNext();

                        if(reader.isCharacters())
                        {
                            ui.jmbgEdit->setText(reader.text().toString());

                        }
                }

                if (reader.name() == "upisan")
                {
                        reader.readNext();

                        if(reader.isCharacters())
                        {
                            if(!reader.text().toString().trimmed().compare("1"))
                            {
                                ui.enrolledButton->setChecked(true);

                                while ( !reader.atEnd() )
                                {
                                    reader.readNext();

                                    if (reader.isStartElement()) {
                                        if (reader.name() == "predmet")
                                        {
                                            reader.readNext();

                                            if(reader.isCharacters())
                                            {
                                                QListWidgetItem *item=new QListWidgetItem(reader.text().toString(), ui.coursesList);
                                                ui.coursesList->setCurrentItem(item);
                                            }
                                        }
                                    }
                                }
                            }
                            else
                                ui.notEnrolledButton->setChecked(true);
                        }
                    }
                }
            }

        reader.clear();
        file.close();
    }
}

//metod koji zapisuje podatke o studentu u tekstualnu datoteku
void studentinfo::writeTxtFile(QString fileName)
{
    QFile file(fileName);

    if(file.open(QIODevice::WriteOnly))  // datoteka se otvara za citanje
    {
        QTextStream t(&file);

        t << ui.nameEdit->text() << endl;
        t << ui.surnameEdit->text() << endl;
        t << ui.indexEdit->text() << endl;
        t << ui.jmbgEdit->text() << endl;

        // ako je student upisan onda ima prijavljenih ispita koji se unose u datoteku
        if(ui.enrolledButton->isChecked())
        {
            t<< "upisan" << endl;

            for(int i=0; i<ui.coursesList->count(); i++)
            {
                t<< ui.coursesList->item(i)->text() << endl;
            }
        }
        else
            t<< "nije upisan" << endl;

        file.close();
    }
}


//metod koji zapisuje podatke o studentu u XML datoteku
void studentinfo::writeXMLFile(QString fileName)
{
    QFile file(fileName);

    if(file.open(QIODevice::WriteOnly))  // datoteka se otvara za citanje
    {
        QXmlStreamWriter writer(&file);

        writer.setAutoFormatting(true);
        writer.writeStartDocument();
        writer.writeStartElement("student");
        writer.writeTextElement("ime", ui.nameEdit->text());
        writer.writeTextElement("prezime", ui.surnameEdit->text());
        writer.writeTextElement("brojIndeksa", ui.indexEdit->text());
        writer.writeTextElement("JMBG", ui.jmbgEdit->text());

        if(ui.enrolledButton->isChecked())
        {
            writer.writeTextElement("upisan", "1");
            for(int i=0; i<ui.coursesList->count(); i++)
            {
                writer.writeTextElement("predmet", ui.coursesList->item(i)->text());
            }
        }
        else
            writer.writeTextElement("upisan", "0");


        writer.writeEndElement(); // student
        writer.writeEndDocument();

    }

    file.close();
}

