#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <exception>
#include <stdexcept>

using namespace std;

typedef unsigned char Bajt;

class NizBajtova
{
public:

    NizBajtova()
    {}

    NizBajtova(const string& s)
    {
        unsigned v=s.length();
        _niz.resize(v);
        for(unsigned i=0; i<v; i++)
            _niz[i]=s[i];
    }

    Bajt operator[](unsigned i) const
    {
        return _niz[i];
    }

    Bajt& operator[](unsigned i)
            {
        return _niz[i];
            }

    void promeniVelicinu(unsigned v)
    {
        _niz.resize(v);
    }

    unsigned velicina() const
    {
        return _niz.size();
    }

    void pisi(ostream& ostr) const
    {
        ostr.write((char*)&_niz[0], velicina());
        if(!ostr)
            throw runtime_error("Nije uspelo pisanje u datoteku!");
    }

    void citaj(istream& istr)
    {
        istr.seekg(0, ios::end);
        unsigned v=istr.tellg();
        istr.seekg(0, ios::beg);
        _niz.resize(v);
        istr.read((char*)&_niz[0], v);
        if(!istr)
            throw runtime_error("Nije uspelo citanje datoteke!");
    }

private:
    vector<Bajt> _niz;
};

class Transformacija
{
public:

    virtual ~Transformacija()
    {}

    virtual void kodiranje(const NizBajtova& org, NizBajtova& rez) const=0;

    virtual  void dekodiranje(const NizBajtova& org, NizBajtova& rez) const=0;

    virtual Transformacija* kopija() const=0;
};


class TransformacijaBajtova : public Transformacija
{
public:
    TransformacijaBajtova(Bajt kod)
    :_kod(kod)
     {}

    void kodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        unsigned vel=org.velicina();
        rez.promeniVelicinu(vel);
        for(unsigned int i=0; i<vel; i++)
            rez[i]=kodiranjeBajta(org[i]);
    }

    void dekodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        unsigned vel=org.velicina();
        rez.promeniVelicinu(vel);
        for(unsigned int i=0; i<vel; i++)
            rez[i]=dekodiranjeBajta(org[i]) ;
    }

    TransformacijaBajtova* kopija() const=0;

protected:

    virtual Bajt kodiranjeBajta(Bajt b) const=0;
    virtual Bajt dekodiranjeBajta(Bajt b) const=0;

    Bajt _kod;

};

class Translacija :public TransformacijaBajtova
{

public:
    Translacija(Bajt kod)
        :TransformacijaBajtova(kod)
         {}

    Translacija* kopija() const
    {
        return new Translacija(_kod);
    }

protected:

    Bajt kodiranjeBajta(Bajt b) const
    {
        return b+_kod;
    }

    Bajt dekodiranjeBajta(Bajt b) const
    {
        return b-_kod;
    }
};

class Rotacija : public TransformacijaBajtova
{
public:
    Rotacija(Bajt kod)
        :TransformacijaBajtova(kod)
         {}

    Rotacija* kopija() const
    {
        return new Rotacija(*this);
    }

protected:

    Bajt kodiranjeBajta(Bajt b) const
    {
        return (b << _kod) | (b >> (8-_kod));
    }

    Bajt dekodiranjeBajta(Bajt b) const
    {
        return (b >> _kod) | (b << (8-_kod));
    }

};

class Zamena : public Transformacija
{
public:
     void kodiranje(const NizBajtova& org, NizBajtova& rez) const
     {
         transformisanje(org, rez);
     }

     void dekodiranje(const NizBajtova& org, NizBajtova& rez) const
     {
         transformisanje(org, rez);
     }

     Zamena* kopija() const
     {
         return new Zamena();
     }

private:
     static void transformisanje(const NizBajtova& org, NizBajtova& rez)
     {
         unsigned vel=org.velicina();
         rez.promeniVelicinu(vel);
         for(unsigned int i=1; i<vel; i+=2)
         {
             rez[i]=org[i-1];
             rez[i-1]=org[i];
         }

         if(vel%2)
             rez[vel-1]=org[vel-1];
     }
};

class EkskluzivnaDisjunkcija : public Transformacija
{
public:
    EkskluzivnaDisjunkcija(const NizBajtova& kod)
    :_kod(kod)
    {
        if(!kod.velicina())
            throw range_error("Duzina kodirajuceg niza mora biti veca od 0 (EkskluzivnaDiskunkcija");
    }

    void kodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        transformisanje(org, rez);
    }

    void dekodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        transformisanje(org, rez);
    }

    EkskluzivnaDisjunkcija* kopija() const
    {
        return new EkskluzivnaDisjunkcija(*this);
    }

private:
    void transformisanje(const NizBajtova& org, NizBajtova& rez) const
    {
        unsigned vel=org.velicina();
        rez.promeniVelicinu(vel);
        unsigned n=_kod.velicina();
        for(unsigned int i=0; i<vel; i++)
            rez[i]=org[i]^_kod[i%n];
    }

    NizBajtova _kod;
};

class EkskluzivnaDisjunkcijaStart : public Transformacija
{
public:
    EkskluzivnaDisjunkcijaStart(unsigned duzinaKoda)
    :_duzinaKoda(duzinaKoda)
     {
        if(!duzinaKoda)
            throw range_error("Duzina kodirajuceg podniza mora biti veca od 0 (EkskluzivnaDiskunkcijaStart");
     }

    void kodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        transformisanje(org, rez);
    }

    void dekodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        transformisanje(org, rez);
    }

    EkskluzivnaDisjunkcijaStart* kopija() const
    {
        return new EkskluzivnaDisjunkcijaStart(*this);
    }

private:
    void transformisanje(const NizBajtova& org, NizBajtova& rez) const
    {
         unsigned vel=org.velicina();
         rez.promeniVelicinu(vel);
         for(unsigned int i=0; i<_duzinaKoda; i++)
             rez[i]=org[i];
         for(unsigned int i=_duzinaKoda; i<vel; i++)
             rez[i]=org[i]^org[i%_duzinaKoda];

    }

    unsigned _duzinaKoda;
};

class SlozenaTransformacija : public Transformacija
{
public:

    SlozenaTransformacija()
    {}

    SlozenaTransformacija(const SlozenaTransformacija& s)
    {
        init(s);
    }

    ~SlozenaTransformacija()
    {
        deinit();
    }

    SlozenaTransformacija& operator= ( const SlozenaTransformacija& s)
    {
        if(this !=&s)
        {
            deinit();
            init(s);
        }

        return *this;
    }

    void dodaj(const Transformacija* t)
    {
        _transformacije.push_back(t);
    }

    void kodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        rez=org;
        for(unsigned i=0; i<_transformacije.size(); i++)
        {
            NizBajtova p=rez;
            _transformacije[i]->kodiranje(p,rez);
        }
    }

    void dekodiranje(const NizBajtova& org, NizBajtova& rez) const
    {
        rez=org;
        unsigned n=_transformacije.size();
        for(unsigned i=0; i<n; i++)
        {
            NizBajtova p=rez;
            _transformacije[n-1-i]->dekodiranje(p,rez);
        }

    }

    SlozenaTransformacija* kopija() const
    {
        return new SlozenaTransformacija(*this);
    }

private:

    void deinit()
    {
        for(unsigned i=0; i<_transformacije.size(); i++)
            delete _transformacije[i];
    }

    void init(const SlozenaTransformacija& s)
    {
        _transformacije.clear();
        for(unsigned i=0; i<s._transformacije.size(); i++)
            _transformacije.push_back(s._transformacije[i]->kopija());
    }

    vector<const Transformacija*> _transformacije;

};

class ProgKoDek
{
public:
    void izvrsi(int argc, char** argv)
    {
        priprema(argc, argv);
        obrada();
    }

private:
    void priprema(int argc, char** argv)
    {
        if(argc<5)
        {
            throw invalid_argument("Nisu navedeni svi neophodni parametri!");
        }

        string smer=argv[1];
        if(smer=="k")
            kodiranje=true;
        else if (smer == "d") {
            kodiranje=false;
        }
        else {
            throw invalid_argument("Tip operacije mora biti 'k' (kodiranje) ili 'd' (dekodiranje)!");
        }

        citanjeTransformacije(argc, argv);

        imeUlazneDatoteke=argv[2];
        imeIzlazneDatoteke=argv[3];

    }

    void citanjeTransformacije(int argc, char** argv)
    {
        for(int i=4; i<argc; i++)
        {
            string param=argv[i];
            Transformacija* t=0;

            if(param=="trans")
            {
                t=new Translacija(intArg(++i, argc, argv, "translacije"));

            }

            else if (param == "rot")
            {
                t=new Rotacija(intArg(++i, argc, argv, "rotacije"));
            }
            else if (param == "zamena") {
                t=new Zamena;
            }
            else if (param == "xor")
            {
                i++;
                if(i>=argc)
                    throw invalid_argument("Nedostaje parametar eks. disjunkcije");

                t=new EkskluzivnaDisjunkcija(NizBajtova(argv[i]));
            }
            else if (param == "xorstart") {
                t=new EkskluzivnaDisjunkcijaStart(intArg(++i, argc, argv, "eks. disj. start"));
            }

            else
                throw invalid_argument("Nepoznato kodiranje!");

            transformacija.dodaj(t);

        }
    }

    static int intArg(int i, int argc, char** argv, string op)
    {
        if(i>=argc)
            throw invalid_argument(string("Nedostaje parametar ") + op + "!");

        int n;
        if(!sscanf(argv[i], "%d", &n))
                    throw invalid_argument(string("Nije ispravan parametar ") + op + "!");
        return n;
    }

    void obrada()
    {
        citanjeDatoteke();
        izvodjenjeOperacije();
        pisanjeDatoteke();
    }

    void citanjeDatoteke()
    {
        ifstream uDat(imeUlazneDatoteke, ios::binary);
        if(!uDat)
        {
            throw runtime_error("Nije uspelo otvaranje ulazne datoteke!");
        }
        originalniNizBajtova.citaj(uDat);

    }

    void pisanjeDatoteke()
    {
        ofstream iDat(imeIzlazneDatoteke, ios::binary);
        if(!iDat)
        {
            throw runtime_error("Nije uspelo otvaranje izlazne datoteke!");
        }

        transformisaniNizBajtova.pisi(iDat);
}

    void izvodjenjeOperacije()
    {
        if(kodiranje)
        {
            transformacija.kodiranje(originalniNizBajtova,transformisaniNizBajtova);
        }
        else
        {
            transformacija.dekodiranje(originalniNizBajtova, transformisaniNizBajtova);
        }
    }

    bool kodiranje;
    string imeUlazneDatoteke;
    string imeIzlazneDatoteke;
    NizBajtova originalniNizBajtova;
    NizBajtova transformisaniNizBajtova;
    SlozenaTransformacija transformacija;
};


int main(int argc, char** argv)
{
    try
    {
        ProgKoDek program;
        program.izvrsi(argc, argv);
    }
    catch (invalid_argument& e) {
        cerr << "Neispravna upotreba programa" << endl;
        cerr << e.what() << endl;
        cerr << "Kodiranje (k|d) <ulazna dat.> <izlazna dat> <transformacije>" << endl;
        return 1;
    }
    catch (exception& e) {
        cerr << "Greska: " << e.what() << endl;
        return 1;
    }
    return 0;

}

